@@ -8,7 +8,7 @@ import {
 import { components } from "./store.js";
 import { send } from "./messageSender.js";
 import morphdom from "./morphdom/2.6.1/morphdom.js";
-import { $, contains, hasValue, isEmpty, isFunction, walk } from "./utils.js";
+import { $, hasValue, isEmpty, isFunction, walk } from "./utils.js";
 
 /**
  * Encapsulate component.
@@ -20,7 +20,7 @@ export class Component {
     this.key = args.key;
     this.messageUrl = args.messageUrl;
     this.csrfTokenHeaderName = args.csrfTokenHeaderName;
-    this.data = args.data;
+    this.data = args.data || {};
     this.syncUrl = `${this.messageUrl}/${this.name}`;
 
     this.document = args.document || document;
@@ -49,6 +49,8 @@ export class Component {
     this.init();
     this.refreshEventListeners();
     this.initPolling();
+
+    this.callCalls(args.calls);
   }
 
   /**
@@ -113,6 +115,20 @@ export class Component {
     return parentComponent;
   }
 
+  /**
+   * Call JavaScript functions on the `window`.
+   * @param {Array} calls A list of objects that specify the methods to call.
+   *
+   * `calls`: [{"fn": "someFunctionName"},]
+   */
+  callCalls(calls) {
+    calls = calls || [];
+
+    calls.forEach((call) => {
+      this.window[call.fn](...call.args);
+    });
+  }
+
   /**
    * Sets event listeners on unicorn elements.
    */
@@ -245,6 +261,32 @@ export class Component {
     }
   }
 
+  /**
+   * Check to see if the poll is disabled.
+   */
+  isPollEnabled() {
+    if (!this.poll.disable) {
+      if (hasValue(this.poll.disableData)) {
+        if (this.poll.disableData.startsWith("!")) {
+          // Manually negate this and re-negate it after the check
+          this.poll.disableData = this.poll.disableData.slice(1);
+
+          if (this.data[this.poll.disableData]) {
+            return true;
+          }
+
+          this.poll.disableData = `!${this.poll.disableData}`;
+        } else if (!this.data[this.poll.disableData]) {
+          return true;
+        }
+      } else {
+        return true;
+      }
+    }
+
+    return false;
+  }
+
   /**
    * Sets up polling if it is defined on the component's root.
    */
@@ -271,12 +313,15 @@ export class Component {
 
       this.poll.partial = rootElement.partial;
 
-      // Call the method once before the timer starts
-      this.callMethod(
-        this.poll.method,
-        this.poll.partial,
-        this.handlePollError
-      );
+      if (this.isPollEnabled()) {
+        // Call the method once before the timer starts
+        this.callMethod(
+          this.poll.method,
+          this.poll.partial,
+          this.handlePollError
+        );
+      }
+
       this.startPolling();
     }
   }
@@ -286,35 +331,12 @@ export class Component {
    */
   startPolling() {
     this.poll.timer = setInterval(() => {
-      if (!this.poll.disable) {
-        if (hasValue(this.poll.disableData)) {
-          if (this.poll.disableData.startsWith("!")) {
-            // Manually negate this and re-negate it after the check
-            this.poll.disableData = this.poll.disableData.slice(1);
-
-            if (this.data[this.poll.disableData]) {
-              this.callMethod(
-                this.poll.method,
-                this.poll.partial,
-                this.handlePollError
-              );
-            }
-
-            this.poll.disableData = `!${this.poll.disableData}`;
-          } else if (!this.data[this.poll.disableData]) {
-            this.callMethod(
-              this.poll.method,
-              this.poll.partial,
-              this.handlePollError
-            );
-          }
-        } else {
-          this.callMethod(
-            this.poll.method,
-            this.poll.partial,
-            this.handlePollError
-          );
-        }
+      if (this.isPollEnabled()) {
+        this.callMethod(
+          this.poll.method,
+          this.poll.partial,
+          this.handlePollError
+        );
       }
     }, this.poll.timing);
   }
