@@ -7,7 +7,7 @@ app_name = "django_unicorn"
 
 
 urlpatterns = (
-    re_path("message/(?P<component_name>[\w/\.]+)", views.message, name="message"),
+    re_path("message/(?P<component_name>[\w/\.-]+)", views.message, name="message"),
     path(
         "message", views.message, name="message"
     ),  # Only here to build the correct url in scripts.html
