@@ -1,5 +1,6 @@
 import copy
 import logging
+from dataclasses import is_dataclass
 from functools import wraps
 from typing import Any, Dict, List, Union, get_type_hints
 
@@ -11,6 +12,7 @@ from django.views.decorators.http import require_POST
 
 import orjson
 from bs4 import BeautifulSoup
+from cachetools.lru import LRUCache
 
 from .call_method_parser import InvalidKwarg, parse_call_method_name, parse_kwarg
 from .components import UnicornField, UnicornView
@@ -26,7 +28,14 @@ logger = logging.getLogger(__name__)
 logger.setLevel(logging.DEBUG)
 
 
+type_hints_cache = LRUCache(maxsize=100)
+
+
 def handle_error(view_func):
+    """
+    Returns a JSON response with an error if necessary.
+    """
+
     def wrapped_view(*args, **kwargs):
         try:
             return view_func(*args, **kwargs)
@@ -38,6 +47,34 @@ def handle_error(view_func):
     return wraps(view_func)(wrapped_view)
 
 
+def _get_type_hints(obj):
+    """
+    Get type hints from an object. These get cached in a local memory cache for quicker look-up later.
+
+    Returns:
+        An empty dictionary if no type hints can be retrieved.
+    """
+    try:
+        if obj in type_hints_cache:
+            return type_hints_cache[obj]
+    except TypeError:
+        # Ignore issues with checking for an object in the cache, e.g. when a Django model is missing a PK
+        pass
+
+    try:
+        type_hints = get_type_hints(obj)
+
+        # Cache the type hints just in case
+        type_hints_cache[obj] = type_hints
+
+        return type_hints
+    except TypeError:
+        #  Return an empty dictionary when there is a TypeError. From `get_type_hints`: "TypeError is
+        # raised if the argument is not of a type that can contain annotations, and an empty dictionary
+        # is returned if no annotations are present"
+        return {}
+
+
 @timed
 def _is_component_field_model_or_unicorn_field(
     component_or_field: Union[UnicornView, UnicornField, Model], name: str,
@@ -66,7 +103,7 @@ def _is_component_field_model_or_unicorn_field(
     component_type_hints = {}
 
     try:
-        component_type_hints = get_type_hints(component_or_field)
+        component_type_hints = _get_type_hints(component_or_field)
 
         if name in component_type_hints:
             is_subclass_of_model = issubclass(component_type_hints[name], Model)
@@ -81,7 +118,7 @@ def _is_component_field_model_or_unicorn_field(
                 if is_subclass_of_model or is_subclass_of_unicorn_field:
                     field = component_type_hints[name]()
                     setattr(component_or_field, name, field)
-    except TypeError as e:
+    except TypeError:
         pass
 
     return is_subclass_of_model or is_subclass_of_unicorn_field
@@ -115,6 +152,19 @@ def _set_property_from_data(
         else:
             _set_property_from_data(field, field.name, value)
     else:
+        type_hints = _get_type_hints(component_or_field)
+
+        if name in type_hints:
+            # Construct the specified type by passing the value in
+            # Usually the value will be a string (because it is coming from JSON)
+            # and basic types can be constructed by passing in a string,
+            # i.e. int("1") or float("1.1")
+
+            if is_dataclass(type_hints[name]):
+                value = type_hints[name](**value)
+            else:
+                value = type_hints[name](value)
+
         if hasattr(component_or_field, "_set_property"):
             # Can assume that `component_or_field` is a component
             component_or_field._set_property(name, value)
@@ -434,6 +484,8 @@ def _process_component_request(
         else:
             raise UnicornViewError(f"Unknown action_type '{action_type}'")
 
+    component.complete()
+
     # Re-load frontend context variables to deal with non-serializable properties
     component_request.data = orjson.loads(component.get_frontend_context_variables())
 
@@ -454,6 +506,8 @@ def _process_component_request(
             component.validate(model_names=list(updated_data.keys()))
 
     rendered_component = component.render()
+    component.rendered(rendered_component)
+
     partial_doms = []
 
     if partials and all(partials):
@@ -501,6 +555,7 @@ def _process_component_request(
         "id": component_request.id,
         "data": updated_data,
         "errors": component.errors,
+        "calls": component.calls,
         "checksum": generate_checksum(orjson.dumps(component_request.data)),
     }
 
@@ -539,6 +594,7 @@ def _process_component_request(
 
         if not partial_doms:
             parent_dom = parent_component.render()
+            component.parent_rendered(parent_dom)
 
             parent.update(
                 {
